@@ -1,7 +1,7 @@
 /*
  * Calcoo: b_display.c
  *
- * Copyright (C) 2001, 2002 Alexei Kaminski
+ * Copyright (C) 2001, 2002, 2005 Alexei Kaminski
  *
  * display creation and
  * functions that facilitate display functionality
@@ -76,43 +76,41 @@ void create_display(int display_number,
 	y1 = y + margin_vert;
 	display_exp_indent = cell_width * (INPUT_LENGTH + 1) + dot_width;
 
+	display->x1 = x1;
+	display->y1 = y1;
+	display->display_exp_indent = display_exp_indent;
+
+	display->digit_xpm = digit_xpm;
+	display->cell_width = cell_width;
+	display->dot_offset_vert = dot_offset_vert;
+	display->dot_width = dot_width;
+	display->margin_vert = margin_vert;
+	display->margin_horiz = margin_horiz;
+	display->error_indent = error_indent;
+	display->tick_offset_vert = tick_offset_vert;
+	display->tick_offset_horiz = tick_offset_horiz;
+
 	/* Things to show */
 	for (i = 0; i < INPUT_LENGTH; i++) 
 		for (n = 0; n < BASE; n++) 
-			create_n_put_pixmap(&display->int_digits[i][n],
-					    x1 + cell_width * (i + 1), y1,
-					    digit_xpm[n]);
+			display->int_digits[i][n] = NULL;
 
 	for (i = 0; i < INPUT_LENGTH; i++) 
 		for (n = 0; n < BASE; n++) 
-			create_n_put_pixmap(&display->frac_digits[i][n],
-					    x1 + dot_width 
-					       + cell_width * (i + 1), y1,
-					    digit_xpm[n]);
+			display->frac_digits[i][n] = NULL;
 
 	for (i = 0; i < EXP_INPUT_LENGTH; i++) 
 		for (n = 0; n < BASE; n++) 
-			create_n_put_pixmap(&display->exp_digits[i][n],
-					    x1 + display_exp_indent
-			  		       + cell_width * (i + 2), y1,
-					    digit_xpm[n]);
+			display->exp_digits[i][n] = NULL;
 
 	for (i = 0; i < INPUT_LENGTH; i++) 
-		create_n_put_pixmap(&display->dot[i],
-				    x1 + cell_width * (i + 2), 
-				    y1 + dot_offset_vert,
-				    digit_xpm[D_DOT]);
+		display->dot[i] = NULL;
 
 	for (i = 0; i < INPUT_LENGTH; i++) 
-		create_n_put_pixmap(&display->minus_sign[i],
-				    x1 + cell_width * i, y1,
-				    digit_xpm[D_MINUS]);
+		display->minus_sign[i] = NULL;
 
 	for (i = 0; i < INPUT_LENGTH; i++) 
-		create_n_put_pixmap(&display->int_ticks[i],
-				    x1 + cell_width * i + tick_offset_horiz,
-				    y1 + tick_offset_vert,
-				    digit_xpm[D_TICK]);
+		display->int_ticks[i] = NULL;
 
 	for (i = 0; i < INPUT_LENGTH; i++) 
 		create_n_put_pixmap(&display->frac_ticks[i],
@@ -121,24 +119,11 @@ void create_display(int display_number,
 				    y1 + tick_offset_vert,
 				    digit_xpm[D_TICK]);
 
+	display->exp_minus_sign = NULL;
+	display->exp_plus_sign = NULL;
+	display->E = NULL;
 
-	create_n_put_pixmap(&display->exp_minus_sign, 
-			    x1 + display_exp_indent + cell_width, y1,
-			    digit_xpm[D_MINUS]);
-		       
-	create_n_put_pixmap(&display->exp_plus_sign, 
-			    x1 + display_exp_indent + cell_width, y1,
-			    digit_xpm[D_PLUS]);
-
-	create_n_put_pixmap(&display->E, 
-			    x1 + display_exp_indent, y1,
-			    digit_xpm[D_E]);
-
-	create_n_put_pixmap(&display->error_message, 
-			    x1 + error_indent, y1,
-			    digit_xpm[D_OVERFLOW]);
-
-	clear_display(display_number);
+	display->error_message = NULL;
 }
 
 void clear_display(int display_number)
@@ -151,85 +136,192 @@ void clear_display(int display_number)
 
 	for (i = 0; i < INPUT_LENGTH; i++) 
 		for (n = 0; n < BASE; n++) 
-			gtk_widget_hide(display->int_digits[i][n]);
+			if ( NULL != display->int_digits[i][n] )
+				gtk_widget_hide(display->int_digits[i][n]);
 
 	for (i = 0; i < INPUT_LENGTH; i++) 
 		for (n = 0; n < BASE; n++) 
-			gtk_widget_hide(display->frac_digits[i][n]);
+			if ( NULL != display->frac_digits[i][n] )
+				gtk_widget_hide(display->frac_digits[i][n]);
 
 	for (i = 0; i < EXP_INPUT_LENGTH; i++) 
 		for(n = 0; n < BASE; n++) 
-			gtk_widget_hide(display->exp_digits[i][n]);
-
+			if ( NULL != display->exp_digits[i][n] )
+				gtk_widget_hide(display->exp_digits[i][n]);
 
 	for(i = 0; i < INPUT_LENGTH; i++) 
-		gtk_widget_hide(display->dot[i]);
+		if ( NULL != display->dot[i] )
+			gtk_widget_hide(display->dot[i]);
 
 	for(i = 0; i < INPUT_LENGTH; i++) 
-		gtk_widget_hide(display->minus_sign[i]);
+		if ( NULL != display->minus_sign[i] )
+			gtk_widget_hide(display->minus_sign[i]);
+	
+	for(i = 0; i < INPUT_LENGTH; i++) 
+		if ( NULL != display->int_ticks[i] )
+			gtk_widget_hide(display->int_ticks[i]);
 
 	for(i = 0; i < INPUT_LENGTH; i++) 
-		gtk_widget_hide(display->int_ticks[i]);
+		if ( NULL != display->frac_ticks[i] )
+			gtk_widget_hide(display->frac_ticks[i]);
 
-	for(i = 0; i < INPUT_LENGTH; i++) 
-		gtk_widget_hide(display->frac_ticks[i]);
+	if ( NULL != display->exp_minus_sign )
+		gtk_widget_hide(display->exp_minus_sign);
+	if ( NULL != display->exp_plus_sign )
+		gtk_widget_hide(display->exp_plus_sign);
+	if ( NULL != display->E )
+		gtk_widget_hide(display->E);
 
-	gtk_widget_hide(display->exp_minus_sign);
-	gtk_widget_hide(display->exp_plus_sign);
-	gtk_widget_hide(display->E);
-	gtk_widget_hide(display->error_message);
+	if ( NULL != display->error_message )
+		gtk_widget_hide(display->error_message);
 }
 
 void show_int_digit(int d, int i, int n)
 {
+	t_body_display *display;
+	display = body->display[d];
+	if ( NULL == display->int_digits[i][n] )
+		create_n_put_pixmap(&display->int_digits[i][n],
+				    display->x1 
+				        + display->cell_width * (i + 1), 
+				    display->y1,
+				    display->digit_xpm[n]);
+
 	gtk_widget_show(body->display[d]->int_digits[i][code_to_digit(n)]);
 }
 
 void show_frac_digit(int d, int i, int n)
 {
+	t_body_display *display;
+	display = body->display[d];
+	if ( NULL == display->frac_digits[i][n] )
+		create_n_put_pixmap(&display->frac_digits[i][n],
+				    display->x1 + display->dot_width 
+				        + display->cell_width * (i + 1), 
+				    display->y1,
+				    display->digit_xpm[n]);
+
 	gtk_widget_show(body->display[d]->frac_digits[i][code_to_digit(n)]);
 }
 
 void show_exp_digit(int d, int i, int n)
 {
+	t_body_display *display;
+	display = body->display[d];
+	if ( NULL == display->exp_digits[i][n] )
+		create_n_put_pixmap(&display->exp_digits[i][n],
+				    display->x1 + display->display_exp_indent
+				        + display->cell_width * (i + 2), 
+				    display->y1,
+				    display->digit_xpm[n]);
+
 	gtk_widget_show(body->display[d]->exp_digits[i][code_to_digit(n)]);
 }
 
+void show_dot(int d, int i)
+{	
+	t_body_display *display;
+	display = body->display[d];
+	if ( NULL == display->dot[i] )
+		create_n_put_pixmap(&display->dot[i],
+				    display->x1 
+				        + display->cell_width * (i + 2), 
+				    display->y1 + display->dot_offset_vert,
+				    display->digit_xpm[D_DOT]);
+
+	gtk_widget_show(body->display[d]->dot[i]);
+}
+
 void show_minus(int d, int i)
 {
+	t_body_display *display;
+	display = body->display[d];
+	if ( NULL == display->minus_sign[i] )
+		create_n_put_pixmap(&display->minus_sign[i],
+				    display->x1 + display->cell_width * i, 
+				    display->y1,
+				    display->digit_xpm[D_MINUS]);
+
 	gtk_widget_show(body->display[d]->minus_sign[i]);
 }
 
 void show_int_tick(int d, int i)
 {
+	t_body_display *display;
+	display = body->display[d];
+	if ( NULL == display->int_ticks[i] )
+		create_n_put_pixmap(&display->int_ticks[i],
+				    display->x1 + display->cell_width * i 
+				        + display->tick_offset_horiz,
+				    display->y1 + display->tick_offset_vert,
+				    display->digit_xpm[D_TICK]);
+
 	gtk_widget_show(body->display[d]->int_ticks[i]);
 }
 
 void show_frac_tick(int d, int i)
 {
+	t_body_display *display;
+	display = body->display[d];
+	if ( NULL == display->frac_ticks[i] )
+		create_n_put_pixmap(&display->frac_ticks[i],
+				    display->x1 + display->cell_width * i 
+				        + display->dot_width 
+				        + display->tick_offset_horiz,
+				    display->y1 + display->tick_offset_vert,
+				    display->digit_xpm[D_TICK]);
+
 	gtk_widget_show(body->display[d]->frac_ticks[i]);
 }
 
 void show_E_and_exp_sign(int d, int exp_sign)
 {
+	t_body_display *display;
+	display = body->display[d];
+	if ( NULL == display->E )
+		create_n_put_pixmap(&display->E, 
+				    display->x1 + display->display_exp_indent, 
+ 				    display->y1,
+				    display->digit_xpm[D_E]);
+
 	gtk_widget_show(body->display[d]->E);
+
 	switch (exp_sign) {
 	case SIGN_PLUS:  
+		if ( NULL == display->exp_plus_sign )
+			create_n_put_pixmap(&display->exp_plus_sign, 
+					    display->x1 
+					        + display->display_exp_indent 
+					        + display->cell_width, 
+					    display->y1,
+					    display->digit_xpm[D_PLUS]);
+
 		gtk_widget_show(body->display[d]->exp_plus_sign);
 		break;
 	case SIGN_MINUS: 
+		if ( NULL == display->exp_minus_sign )
+			create_n_put_pixmap(&display->exp_minus_sign, 
+					    display->x1 
+					        + display->display_exp_indent 
+					        + display->cell_width, 
+					    display->y1,
+					    display->digit_xpm[D_MINUS]);
+
 		gtk_widget_show(body->display[d]->exp_minus_sign);
 		break;
 	}
 }
 
-void show_dot(int d, int i)
-{
-	gtk_widget_show(body->display[d]->dot[i]);
-}
-
 void show_error_message(int d)
 {
+	t_body_display *display;
+	display = body->display[d];
+	if ( NULL == display->error_message )
+		create_n_put_pixmap(&display->error_message, 
+				    display->x1 + display->error_indent, 
+				    display->y1,
+				    display->digit_xpm[D_OVERFLOW]);
+
 	gtk_widget_show(body->display[d]->error_message);
 	strcpy(body->char_display,"error");
 }
