@@ -1,7 +1,7 @@
 /*
  * Calcoo: gtkaux.c
  *
- * Copyright (C) 2001, 2002 Alexei Kaminski
+ * Copyright (C) 2001, 2002, 2005 Alexei Kaminski
  *
  * auxiliary functions for operations with gtk widgets
  * no functions related to specific calcoo tasks
@@ -87,10 +87,12 @@ void create_n_put_pixmap(GtkWidget **pixmap_widget, int x, int y,
 	gtk_fixed_put(GTK_FIXED(body->fixer), *pixmap_widget, x, y);
 }
 
-void own_selection(GtkWidget *widget)
+void own_selection( GtkWidget *widget,
+		    gint      *have_selection )
 {
-	gtk_selection_owner_set (widget, GDK_SELECTION_PRIMARY, 
-				 GDK_CURRENT_TIME);
+	*have_selection = gtk_selection_owner_set (body->selection_widget,
+						   GDK_SELECTION_PRIMARY,
+						   GDK_CURRENT_TIME);
 }
 
 void set_selection( GtkWidget        *widget, 
@@ -99,18 +101,35 @@ void set_selection( GtkWidget        *widget,
 		    guint             time_stamp,
 		    gpointer          data )
 {
-	gtk_selection_data_set (selection_data, GDK_SELECTION_TYPE_STRING, 8, 
-				body->char_display, 
-				strlen(body->char_display));
+	gtk_selection_data_set (selection_data, 
+				GDK_SELECTION_TYPE_STRING, 
+				8, 
+ 				body->char_display,  
+ 				strlen(body->char_display)
+		); 
 }
 
-void request_selection(GtkWidget *widget)
+gboolean selection_clear( GtkWidget         *widget,
+			  GdkEventSelection *event,
+			  gint              *have_selection )
 {
-	static GdkAtom atomos = GDK_NONE;
-        if(atomos == GDK_NONE)
-                atomos = gdk_atom_intern("STRING", FALSE);
-        gtk_selection_convert(widget, GDK_SELECTION_PRIMARY, atomos, 
-			      GDK_CURRENT_TIME);
+	*have_selection = FALSE;
+	return TRUE;
+}
+
+void request_selection( GtkWidget *widget,
+			       gpointer data )
+{
+  static GdkAtom targets_atom = GDK_NONE;
+  GtkWidget *window = (GtkWidget *)data;	
+
+  /* Get the atom corresponding to the string "TARGETS" */
+  if (targets_atom == GDK_NONE)
+    targets_atom = gdk_atom_intern ("STRING", FALSE);
+
+  /* And request the "TARGETS" target for the primary selection */
+  gtk_selection_convert (window, GDK_SELECTION_PRIMARY, targets_atom,
+			 GDK_CURRENT_TIME);
 }
 
 void get_selection(GtkWidget *widget, GtkSelectionData *data, gpointer val)
